#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from zope.container.interfaces import IContained

# import packages
from onf_website.reference.forest.schema import ForestsListField
from onf_website.reference.orga.schema import StructuresListField
from pyams_gis.schema import GeoPointZField, GeoPointField, GeoAreaField
from zope.interface import Interface, Attribute
from zope.schema import Bool, Int, Set, Choice, Text, TextLine

from onf_website import _


LOCATION_INFO_KEY = 'onf_website.location'


class ILocationInfo(IContained):
    """Location information interface"""

    can_inherit = Attribute("Can inherit location?")

    inherit = Bool(title=_("Inherit location?"),
                   required=False,
                   default=True)

    no_inherit = Bool(title=_("Don't inherit location?"),
                      required=False,
                      default=False)

    countries = Set(title=_("Countries"),
                    value_type=Choice(vocabulary='ONF countries'),
                    required=False)

    countries_index = Attribute("Countries list used for indexing")

    departments = Set(title=_("Departments"),
                      value_type=Choice(vocabulary='ONF departments'),
                      required=False)

    departments_index = Attribute("Departments list used for indexing")

    structures = StructuresListField(title=_("Structures"),
                                     required=False)

    address = Text(title=_("Address"),
                   required=False)

    postal_code = TextLine(title=_("Postal code"),
                           required=False)

    city = TextLine(title=_("City"),
                    required=False)

    situation = TextLine(title=_("Relative situation"),
                         description=_("For example: 25km north-west from Paris"),
                         required=False)

    natural_region = TextLine(title=_("Natural region"),
                              required=False)

    forests = ForestsListField(title=_("Managed forests"),
                               required=False)

    forests_label = TextLine(title=_("Substitution label"),
                             required=False)

    gps_location = GeoPointZField(title=_("Geographic position"),
                                  required=False)


class ILocationTarget(Interface):
    """Location marker interface"""


VIEW_LOCATION_SETTINGS_KEY = 'onf_website.view.location'


class IViewLocationSettings(Interface):
    """View location settings"""

    select_context_countries = Bool(title=_("Select context countries?"),
                                    description=_("If 'yes', countries will be extracted from context location"),
                                    required=True,
                                    default=False)

    countries = Set(title=_("Other countries"),
                    value_type=Choice(vocabulary='ONF countries'),
                    required=False)

    def get_countries(self, context):
        """Get all location countries for given context"""

    select_context_departments = Bool(title=_("Select context departments?"),
                                      description=_("If 'yes', departments will be extracted from context location"),
                                      required=True,
                                      default=False)

    departments = Set(title=_("Other departments"),
                      value_type=Choice(vocabulary='ONF departments'),
                      required=False)

    def get_departments(self, context):
        """Get all departments for given context"""

    select_context_structures = Bool(title=_("Select context structures?"),
                                     description=_("If 'yes', structures will be extracted from context location"),
                                     required=True,
                                     default=False)

    structures = StructuresListField(title=_("Other structures"),
                                     required=False)

    def get_structures(self, context):
        """Get all structures for given context"""

    select_context_postal_code = Bool(title=_("Select context postal code?"),
                                      description=_("If 'yes', postal code will be extracted from context location"),
                                      required=True,
                                      default=False)

    postal_code = TextLine(title=_("Other postal code"),
                           required=False)

    def get_postal_codes(self, context):
        """Get all postal codes for given context"""

    select_context_city = Bool(title=_("Select context city?"),
                               description=_("If 'yes', city will be extracted from context location"),
                               required=True,
                               default=False)

    city = TextLine(title=_("Other city"),
                    required=False)

    def get_cities(self, context):
        """Get all cities for given context"""

    select_context_forests = Bool(title=_("Select context forests?"),
                                  description=_("If 'yes', forests will be extracted from context location"),
                                  required=True,
                                  default=False)

    forests = ForestsListField(title=_("Other forests"),
                               required=False)

    def get_forests(self, context):
        """Get all forests for given context"""

    select_context_gps_location = Bool(title=_("Select context GPS location?"),
                                       description=_("If 'yes', GPS coordinates will be extracted "
                                                     "from context location"),
                                       required=True,
                                       default=False)

    gps_location = GeoPointField(title=_("Geographic position"),
                                 required=False)

    gps_distance = Int(title=_("GPS distance from position"),
                       description=_("Distance from GPS location, given in kilometers; "
                                     "requires ElasticSearch extension"),
                       required=False)

    gps_area = GeoAreaField(title=_("GPS area rect"),
                            description=_("Area from which items should be selected; requires ElasticSearch extension"),
                            required=False)

    def get_gps_location(self, context):
        """Get GPS area for given context"""
